from palett.convert.utils.hex import dilute_hex


def hex_int(hex_color: str):
    if hex_color[0] == '#': hex_color = hex_color[1:]
    if len(hex_color) < 6: hex_color = dilute_hex(hex_color)
    return int(hex_color, 16)


'''
@param {string} hex
@returns {number}
'''



def bound(rgb):
    r, g, b = rgb
    ma, mi = r, r
    if g > r:
        ma = g
    else:
        mi = g
    if b > ma:
        ma = b
    if b < mi:
        mi = b
    return (
        ma,
        ma + mi,
        ma - mi
    )  # max, sum, dif



from palett.structs import Card

indigo = Card(
    base='#3F51B5',
    lighten_5='#E8EAF6',
    lighten_4='#C5CAE9',
    lighten_3='#9FA8DA',
    lighten_2='#7986CB',
    lighten_1='#5C6BC0',
    darken_1='#3949AB',
    darken_2='#303F9F',
    darken_3='#283593',
    darken_4='#1A237E',
    accent_1='#8C9EFF',
    accent_2='#536DFE',
    accent_3='#3D5AFE',
    accent_4='#304FFE'
)



from aryth import constraint

from palett import hex_hsl, hsl_hex, rgb_hsl, hsl_rgb


def hex_toner(hex_color, dh, ds, dl):
    hsl = hex_hsl(hex_color)
    hsl = hsl_toner(hsl, dh, ds, dl)
    return hsl_hex(hsl)


def rgb_toner(rgb, dh, ds, dl):
    hsl = rgb_hsl(rgb)
    hsl = hsl_toner(hsl, dh, ds, dl)
    return hsl_rgb(hsl)


def hsl_toner(hsl, dh, ds, dl):
    hue, saturation, lightness = hsl
    hue = constraint(hue + dh, 0, 360)
    saturation = constraint(saturation + ds, 0, 100)
    lightness = constraint(lightness + dl, 0, 100)
    return hue, saturation, lightness



from palett.fluo.fluo_vector import fluo_vector
from veho.vector import mapper as map_vector, mutate as mut_vector


def fluo_rowwise(mx, presets, effects=None, colorant=False, mutate=False):
    mapper = mut_vector if mutate else map_vector
    return mapper(mx, lambda row: fluo_vector(row, presets, effects, colorant, mutate))



RED = 'red'
PINK = 'pink'
PURPLE = 'purple'
DEEPPURPLE = 'deep_purple'
INDIGO = 'indigo'
BLUE = 'blue'
LIGHTBLUE = 'light_blue'
CYAN = 'cyan'
TEAL = 'teal'
GREEN = 'green'
LIGHTGREEN = 'light_green'
LIME = 'lime'
YELLOW = 'yellow'
AMBER = 'amber'
ORANGE = 'orange'
DEEPORANGE = 'deep_orange'
BROWN = 'brown'
BLUEGREY = 'blue_grey'
GREY = 'grey'



from crostab import Crostab
from crostab.convert import samples_to_crostab
from ject import oneself
from texting import CO

import palett.cards as cards
from palett.convert import hex_rgb, hex_hsl
from palett.dye import DyeFactory
from palett.enum.color_groups import entire as entire_colors
from palett.enum.color_degrees import entire as entire_degrees
from palett.enum.color_spaces import HEX, RGB, HSL
from palett.enum.font_effects import INVERSE


def module_to_dict(module): return {k: getattr(module, k) for k in dir(module) if not k.startswith('_')}


card_collection = module_to_dict(cards)


def palett_crostab(
        space=HEX,
        colors=entire_colors,
        degrees=entire_degrees,
        dyed=False
):
    crostab: Crostab = samples_to_crostab(card_collection, side=colors, head=degrees).transpose()
    if space != HEX:
        crostab.mutate(hex_rgb if space == RGB else hex_hsl if space == HSL else oneself)
    if dyed:
        dye_factory = DyeFactory(space, INVERSE)
        crostab.mutate(lambda hex_color: dye_factory(hex_color)(hex_color)) \
            if space == HEX \
            else crostab.mutate(lambda xyz: dye_factory(xyz)(CO.join([f'{v:3.0f}' for v in xyz])))
    return crostab.mutate_head(lambda tx: str(tx).replace('light', 'l').replace('deep', 'd'))



from palett.convert import hex_int
from palett.utils.ansi import FORE, SC


def hex_ansi(hex_color):
    val = hex_int(hex_color)
    return FORE + SC + str(val >> 16 & 0xFF) + SC + str(val >> 8 & 0xFF) + SC + str(val & 0xFF)



from aryth.bound.matrix.duobound import duobound
from aryth.bound.matrix.solebound import solebound
from palett.projector import ProjectorFactory
from palett.projector.utils.preset_to_flat import preset_to_flat
from veho.matrix import size
from veho.matrix.enumerate import mapper as mapper_fn, mutate as mutate_fn

SLICE = 'slice'


def fluo_pointwise(mx, presets, effects=None, colorant=False, mutate=False):
    h, w = size(mx)
    if not h or not w: return [[]]
    if effects is None: effects = ()
    if isinstance(presets, tuple):
        preset_x, preset_y = presets
        info_x, info_y = duobound(mx)
        dye_x, dye_y = (ProjectorFactory(info_x, preset_x, *effects),
                        ProjectorFactory(info_y, preset_y, *effects))
        projector = (to_colorant if colorant else to_pigment)(
            info_x, dye_x, info_y, dye_y, preset_to_flat(preset_x)
        )
    else:
        preset = presets
        info = solebound(mx)
        dye = ProjectorFactory(info, preset, *effects)
        projector = (to_colorant if colorant else to_pigment)(
            info, dye, None, None, preset_to_flat(preset)
        )
    mapper = mutate_fn if mutate else mapper_fn
    return mapper(mx, projector)


def to_colorant(mx_x, dye_x, mx_y, dye_y, default_dye):
    return lambda _, i, j: dye_x(x) \
        if (x := mx_x[i][j]) is not None \
        else dye_y(y) \
        if mx_y and (y := mx_y[i][j]) is not None \
        else default_dye


def to_pigment(mx_x, dye_x, mx_y, dye_y, default_dye):
    return lambda some, i, j: dye_x(x)(some) \
        if (x := mx_x[i][j]) is not None \
        else dye_y(y)(some) \
        if mx_y and (y := mx_y[i][j]) is not None \
        else default_dye(some)



from palett.fluo.fluo_vector import fluo_vector
from veho.columns import mapper
from veho.matrix import transpose


def fluo_columnwise(mx, presets, effects=None, colorant=False):
    return transpose(mapper(mx, lambda col: fluo_vector(col, presets, effects, colorant, mutate=True)))



from palett.structs import Card

blue = Card(
    base='#2196F3',
    lighten_5='#E3F2FD',
    lighten_4='#BBDEFB',
    lighten_3='#90CAF9',
    lighten_2='#64B5F6',
    lighten_1='#42A5F5',
    darken_1='#1E88E5',
    darken_2='#1976D2',
    darken_3='#1565C0',
    darken_4='#0D47A1',
    accent_1='#82B1FF',
    accent_2='#448AFF',
    accent_3='#2979FF',
    accent_4='#2962FF'
)



from typing import Tuple

from palett.dye.rgb.rgb_ansi import rgb_ansi
from palett.dye.utils import parse_effects
from palett.utils.ansi import CLR_FORE, L, R, SC


def dye(rgb: Tuple[int, int, int], *effects: str):
    head, tail = parse_effects(effects)
    head += SC + rgb_ansi(rgb)
    tail += SC + CLR_FORE
    return lambda text: L + head + R + str(text) + L + tail + R



from palett.structs import Card

light_blue = Card(
    base='#03A9F4',
    lighten_5='#E1F5FE',
    lighten_4='#B3E5FC',
    lighten_3='#81D4FA',
    lighten_2='#4FC3F7',
    lighten_1='#29B6F6',
    darken_1='#039BE5',
    darken_2='#0288D1',
    darken_3='#0277BD',
    darken_4='#01579B',
    accent_1='#80D8FF',
    accent_2='#40C4FF',
    accent_3='#00B0FF',
    accent_4='#0091EA'
)



from dataclasses import dataclass


@dataclass
class Card:
    base: str
    lighten_5: str
    lighten_4: str
    lighten_3: str
    lighten_2: str
    lighten_1: str
    darken_1: str
    darken_2: str
    darken_3: str
    darken_4: str
    accent_1: str
    accent_2: str
    accent_3: str
    accent_4: str



from palett.convert.utils.hsl import hf

'''
 * @param {number} h
 * @param {number} s
 * @param {number} l
 * @returns {number[]}
'''


def hsl_rgb(hsl):
    hu, sa, li = hsl
    sa /= 100
    li /= 100
    a = sa * min(li, 1 - li)
    r = hf(0, hu, a, li)
    g = hf(8, hu, a, li)
    b = hf(4, hu, a, li)
    return round(r * 0xFF), round(g * 0xFF), round(b * 0xFF)



from palett.convert import hsl_rgb, hex_int
from palett.utils.ansi import FORE, SC


def rgb_ansi(rgb):
    r, g, b = rgb
    return FORE + SC + str(r) + SC + str(g) + SC + str(b)


def hex_ansi(hex_color):
    n = hex_int(hex_color)
    return FORE + SC + str(n >> 16 & 0xFF) + SC + str(n >> 8 & 0xFF) + SC + str(n & 0xFF)


def hsl_ansi(hsl):
    r, g, b = hsl_rgb(hsl)
    return FORE + SC + str(r) + SC + str(g) + SC + str(b)



import collections
from types import MethodType

from palett.dye.color_to_ansi import rgb_ansi, hsl_ansi, hex_ansi
from palett.enum.color_spaces import RGB, HEX, HSL
from palett.utils.ansi import Effects, SC, R, L, CLR_FORE


def assign_effects(self, effects: tuple):
    for effect in effects:
        if effect in Effects and (effect := Effects[effect]):
            self.head += SC + effect[0]
            self.tail += SC + effect[1]
    return self


def enclose(text): return L + text + R


def dye(self, text): return self.head + str(text) + self.tail


Cut = collections.namedtuple('Cut', ['head', 'tail'])


class DyeFactory:
    ansi: callable
    head: str = ''
    tail: str = ''

    def __init__(self, space, *effects):
        self.ansi = rgb_ansi if space == RGB \
            else hex_ansi if space == HEX \
            else hsl_ansi if space == HSL \
            else rgb_ansi
        if len(effects): assign_effects(self, effects)

    def __call__(self, color):
        head = enclose(self.head + SC + self.ansi(color))
        tail = enclose(self.tail + SC + CLR_FORE)
        return MethodType(dye, Cut(head, tail))

    def render(self, color, text):
        head = enclose(self.head + SC + self.ansi(color))
        tail = enclose(self.tail + SC + CLR_FORE)
        return head + str(text) + tail



from palett.structs import Card

pink = Card(
    base='#E91E63',
    lighten_5='#FCE4EC',
    lighten_4='#F8BBD0',
    lighten_3='#F48FB1',
    lighten_2='#F06292',
    lighten_1='#EC407A',
    darken_1='#D81B60',
    darken_2='#C2185B',
    darken_3='#AD1457',
    darken_4='#880E4F',
    accent_1='#FF80AB',
    accent_2='#FF4081',
    accent_3='#F50057',
    accent_4='#C51162'
)



from palett.structs import Card

yellow = Card(
    base='#FFEB3B',
    lighten_5='#FFFDE7',
    lighten_4='#FFF9C4',
    lighten_3='#FFF59D',
    lighten_2='#FFF176',
    lighten_1='#FFEE58',
    darken_1='#FDD835',
    darken_2='#FBC02D',
    darken_3='#F9A825',
    darken_4='#F57F17',
    accent_1='#FFFF8D',
    accent_2='#FFFF00',
    accent_3='#FFEA00',
    accent_4='#FFD600'
)



from intype import is_numeric

from palett.dye import DyeFactory
from palett.enum.color_spaces import HEX
from palett.presets import PLANET
from palett.projector.projector import ProjectorFactory
from palett.projector.utils.bound_to_leap import bound_to_leap


def pigment(bound, preset=PLANET, *effects):
    leap = bound_to_leap(bound)
    default_dye = DyeFactory(HEX, *effects)(preset.na)
    projector = ProjectorFactory(leap, preset, *effects)
    return lambda x: projector(x)(x) if is_numeric(x) else default_dye(x)



from .color_modes import BACK, CLR_BACK, CLR_FORE, FORE
from .console_colors import BLACK, BLUE, CYAN, GREEN, GREY, MAGENTA, RED, WHITE, YELLOW
from .control_codes import CLR_ALL, ESC, L, R, SC
from .effects import BOLD, CLR_BOLD, CLR_INVERSE, CLR_ITALIC, CLR_UNDERLINE, INVERSE, ITALIC, UNDERLINE, Effects



from typing import Generator

from foba.dicts.dict_strings import dict_collection
from pyspare import deco_dict, deco_str

from palett import palett_flopper
from palett.enum.font_effects import BOLD
from palett.structs import Preset


class Says:
    flopper: Generator
    effects: tuple

    def __init__(self, *effects):
        self.flopper = palett_flopper(to=Preset.rand)
        self.effects = effects

    def __call__(self, name):
        return Pal(deco_str(name, presets=next(self.flopper), effects=self.effects))


class Pal:
    name: str

    def __init__(self, name):
        self.name = name

    def __call__(self, content):
        print(f'[{self.name}]', content)


def test():
    name, lex = dict_collection.flop_shuffle(7)
    print(name, deco_dict(lex))
    says = Says(BOLD)
    for key, value in lex.items():
        says(key)(value)


test()



from palett.structs import Card

deep_purple = Card(
    base='#673AB7',
    lighten_5='#EDE7F6',
    lighten_4='#D1C4E9',
    lighten_3='#B39DDB',
    lighten_2='#9575CD',
    lighten_1='#7E57C2',
    darken_1='#5E35B1',
    darken_2='#512DA8',
    darken_3='#4527A0',
    darken_4='#311B92',
    accent_1='#B388FF',
    accent_2='#7C4DFF',
    accent_3='#651FFF',
    accent_4='#6200EA'
)



from palett.convert.utils.rgb import bound, hue

TH = 1000

'''
!dif: dif===0
@param {int} r - [0,255]
@param {int} g - [0,255]
@param {int} b - [0,255]
@returns {[int,float,float]} [Hue([0,360]), Saturation([0,100]), Lightness([0,100])]
'''


def rgb_hsl(rgb):
    r, g, b = rgb
    r /= 255
    g /= 255
    b /= 255
    ma, su, df = bound([r, g, b])
    hu = hue(r, g, b, ma, df) * 60
    sa = 0 if not df else (df / (2 - su)) if su > 1 else df / su
    li = su / 2
    return round(hu), round(sa * TH) / 10.0, round(li * TH) / 10.0



from .convert import \
    hex_hsl, hex_int, hex_rgb, \
    hsl_hex, hsl_rgb, \
    rgb_hex, rgb_hsl, rgb_int
from palett.fluo import \
    fluo_vector, \
    fluo_entries, \
    fluo_matrix, fluo_rowwise, fluo_columnwise, fluo_pointwise
from palett.tabular import palett_crostab
from palett.flopper import palett_flopper
from palett.dye import DyeFactory, dye
from palett.projector import ProjectorFactory, projector



accents = [f'accent_{x}' for x in range(4, 0, -1)]
lightens = [f'lighten_{x}' for x in range(5, 0, -1)]
darkens = [f'darken_{x}' for x in range(1, 5)]
base = ['base']
entire = accents + lightens + base + darkens
readable = accents[-3:] + lightens[-3:] + base



from veho.enum.matrix_directions import COLUMNWISE, POINTWISE, ROWWISE

from palett.fluo.fluo_matrix.fluo_columnwise import fluo_columnwise
from palett.fluo.fluo_matrix.fluo_pointwise import fluo_pointwise
from palett.fluo.fluo_matrix.fluo_rowwise import fluo_rowwise


def fluo_matrix(mx, direct, presets, effects=None, colorant=False, mutate=False):
    if direct == ROWWISE:
        return fluo_rowwise(mx, presets, effects, colorant, mutate)
    if direct == COLUMNWISE:
        return fluo_columnwise(mx, presets, effects, colorant)
    if direct == POINTWISE:
        return fluo_pointwise(mx, presets, effects, colorant, mutate)
    return mx



from palett.structs import Card

lime = Card(
    base='#CDDC39',
    lighten_5='#F9FBE7',
    lighten_4='#F0F4C3',
    lighten_3='#E6EE9C',
    lighten_2='#DCE775',
    lighten_1='#D4E157',
    darken_1='#C0CA33',
    darken_2='#AFB42B',
    darken_3='#9E9D24',
    darken_4='#827717',
    accent_1='#F4FF81',
    accent_2='#EEFF41',
    accent_3='#C6FF00',
    accent_4='#AEEA00'
)



from palett.dye import DyeFactory
from palett.enum.color_spaces import HEX
from palett.structs import Preset


def preset_to_flat(preset: Preset, *effects):
    return DyeFactory(HEX, *effects)(preset.na)



from aryth.structs import Leap

from palett.projector.utils.parse_hsl import parse_hsl


def preset_to_leap(preset):
    if not preset: return None
    max_hsl = parse_hsl(preset.max)
    min_hsl = parse_hsl(preset.min)
    return color_bound(max_hsl, min_hsl)


def color_bound(max_hsl, min_hsl):
    max_h, max_s, max_l = max_hsl
    min_h, min_s, min_l = min_hsl
    return Leap(
        min=min_hsl,
        dif=(max_h - min_h, max_s - min_s, max_l - min_l),
        max=max_hsl
    )



from collections import namedtuple

from palett.dye import DyeFactory
from palett.enum.color_spaces import HSL
from palett.projector.utils import bound_to_leap, leverage, preset_to_leap

Leverage = namedtuple('Leverage', ['min', 'lever', 'base', 'factory'])


class ProjectorFactory:
    floor: float
    lever: tuple
    base: tuple
    factory: DyeFactory

    __slots__ = ('floor', 'lever', 'base', 'factory')

    def __init__(self, bound, preset, *effects):
        value_leap = bound_to_leap(bound)
        color_leap = preset_to_leap(preset)
        self.floor = value_leap.min
        self.lever = leverage(color_leap.dif, value_leap.dif) if value_leap.dif else (0, 0, 0)
        self.base = color_leap.min
        self.factory = DyeFactory(HSL, *effects)
        # if DIF not in value_leap or not value_leap[DIF]:
        #     dye = factory(color_leap[MIN])
        #     return lambda _: dye
        # return MethodType(projector, Leverage(
        #     value_leap[MIN],
        #     leverage(color_leap[DIF], value_leap[DIF]),
        #     color_leap[MIN],
        #     factory
        # ))

    # def __call__(self, bound):
    #     if effects is None: effects = ()
    #     if not bound or not preset: return to_oneself()
    #     factory = DyeFactory(HSL, *effects)
    #     bound, leap = bound_to_leap(bound), preset_to_leap(preset)
    #     if DIF not in bound or not bound[DIF]:
    #         dye = factory(leap[MIN])
    #         return lambda _: dye
    #     return MethodType(projector, Leverage(
    #         bound[MIN],
    #         leverage(leap[DIF], bound[DIF]),
    #         leap[MIN],
    #         factory
    #     ))

    def __call__(self, value):
        floor = self.floor
        lever_h, lever_s, lever_l = self.lever
        base_h, base_s, base_l = self.base
        return self.factory((
            scale(value, floor, lever_h, base_h, 360),
            scale(value, floor, lever_s, base_s, 100),
            scale(value, floor, lever_l, base_l, 100),
        ))


# def projector(conf, x):
#     lever_h, lever_s, lever_l = conf.lever
#     base_h, base_s, base_l = conf.base
#     floor = conf.min
#     return conf.factory((
#         scale(x, floor, lever_h, base_h, 360),
#         scale(x, floor, lever_s, base_s, 100),
#         scale(x, floor, lever_l, base_l, 100),
#     ))


def scale(x, floor, lever, base, ceil): return min((max(x, floor) - floor) * lever + base, ceil)



from palett.structs import Card

green = Card(
    base='#4CAF50',
    lighten_5='#E8F5E9',
    lighten_4='#C8E6C9',
    lighten_3='#A5D6A7',
    lighten_2='#81C784',
    lighten_1='#66BB6A',
    darken_1='#43A047',
    darken_2='#388E3C',
    darken_3='#2E7D32',
    darken_4='#1B5E20',
    accent_1='#B9F6CA',
    accent_2='#69F0AE',
    accent_3='#00E676',
    accent_4='#00C853'
)



from palett.structs import Card

blue_grey = Card(
    base='#607D8B',
    lighten_5='#ECEFF1',
    lighten_4='#CFD8DC',
    lighten_3='#B0BEC5',
    lighten_2='#90A4AE',
    lighten_1='#78909C',
    darken_1='#546E7A',
    darken_2='#455A64',
    darken_3='#37474F',
    darken_4='#263238',
    accent_1='#B7C9D1',
    accent_2='#89A5B3',
    accent_3='#6A8EA0',
    accent_4='#547383'
)



def hue(r, g, b, ma, df):
    if df == 0:
        return 0
    if ma == r:
        return ((g - b) / df + (6 if g < b else 0)) % 6
    if ma == g:
        return (b - r) / df + 2
    if ma == b:
        return (r - g) / df + 4



from palett.dye.rgb.rgb_ansi import rgb_ansi
from palett.dye.utils import br, parse_effects
from palett.utils.ansi import CLR_FORE


def prep_dyer(*effects):
    head, tail = parse_effects(effects)
    return lambda rgb: lambda text: br(head, rgb_ansi(rgb)) + text + br(tail, CLR_FORE)



from .hex_hsl import hex_hsl
from .hex_int import hex_int
from .hex_rgb import hex_rgb
from .hsl_hex import hsl_hex
from .hsl_rgb import hsl_rgb
from .rgb_hex import rgb_hex
from .rgb_hsl import rgb_hsl
from .rgb_int import rgb_int



from palett.structs import Card

purple = Card(
    base='#9C27B0',
    lighten_5='#F3E5F5',
    lighten_4='#E1BEE7',
    lighten_3='#CE93D8',
    lighten_2='#BA68C8',
    lighten_1='#AB47BC',
    darken_1='#8E24AA',
    darken_2='#7B1FA2',
    darken_3='#6A1B9A',
    darken_4='#4A148C',
    accent_1='#EA80FC',
    accent_2='#E040FB',
    accent_3='#D500F9',
    accent_4='#AA00FF'
)



from dataclasses import dataclass
from random import randint

from aryth import constraint

from palett.convert import hsl_hex, hex_hsl
from palett.toner import hsl_toner


@dataclass
class Preset:
    max: str
    min: str
    na: str

    @staticmethod
    def rand(hex_color):
        vale = hex_hsl(hex_color)
        peak = hsl_toner(vale, randint(-12, 12), randint(-5, 10), randint(6, 18))
        hue, saturation, lightness = vale
        na = (
            reverse_hue(hue),
            constraint(saturation - 32, 5, 90),
            constraint(lightness + 24, 40, 96)
        )
        return Preset(hsl_hex(vale), hsl_hex(peak), hsl_hex(na))


def reverse_hue(hue):
    hue += 180
    return hue - 360 if hue > 360 else hue + 360 if hue < 0 else hue



from palett.structs import Card

orange = Card(
    base='#FF9800',
    lighten_5='#FFF3E0',
    lighten_4='#FFE0B2',
    lighten_3='#FFCC80',
    lighten_2='#FFB74D',
    lighten_1='#FFA726',
    darken_1='#FB8C00',
    darken_2='#F57C00',
    darken_3='#EF6C00',
    darken_4='#E65100',
    accent_1='#FFD180',
    accent_2='#FFAB40',
    accent_3='#FF9100',
    accent_4='#FF6D00'
)



MAX = 'max'
MIN = 'min'
DIF = 'dif'


def bound_to_leap(bound):
    if hasattr(bound, MIN):
        if hasattr(bound, DIF):
            pass
        elif hasattr(bound, MAX):
            setattr(bound, DIF, (bound.max or 0) - (bound.min or 0))
    elif hasattr(bound, DIF):
        if hasattr(bound, MAX):
            setattr(bound, MIN, (bound.max or 0) - (bound.dif or 0))
        else:
            setattr(bound, MIN, 0)
    elif hasattr(bound, MAX):
        setattr(bound, MIN, 0)
    else:
        setattr(bound, MIN, 0)
        setattr(bound, DIF, 0)
    return bound



from palett.utils.ansi import SC, effects


def parse_effects(input_effects):
    h, t = '', ''
    for ef in input_effects:
        if ef in effects:
            l, r = effects[ef]
            h += SC + l
            t += SC + r
    return h, t



"""
 * @param {number} n
 * @param {number} hu
 * @param {number} a
 * @param {number} li
 * @returns {number}
"""


def hf(n, hu, a, li):
    k = (n + hu / 30) % 12
    return li - a * max(min(k - 3, 9 - k, 1), -1)



from .aqua import AQUA
from .atlas import ATLAS
from .aurora import AURORA
from .azure import AZURE
from .fresh import FRESH
from .insta import INSTA
from .jungle import JUNGLE
from .lava import LAVA
from .metro import METRO
from .moss import MOSS
from .ocean import OCEAN
from .planet import PLANET
from .pome import POME
from .subtle import SUBTLE
from .viola import VIOLA



from veho.entries import mutazip, unwind, wind

from palett.fluo.fluo_vector import fluo_vector


def fluo_entries(entries, presets, effects=None, colorant=False, mutate=False):
    (keys, items) = unwind(entries)
    fluo_vector(keys, presets, effects, colorant, mutate=True)
    fluo_vector(items, presets, effects, colorant, mutate=True)
    rendered = wind(keys, items)
    return mutazip(entries, rendered, lambda _, b: b) if mutate else rendered



BOLD = '1'
ITALIC = '3'
UNDERLINE = '4'
INVERSE = '7'
CLR_BOLD = '22'
CLR_ITALIC = '23'
CLR_UNDERLINE = '24'
CLR_INVERSE = '27'

Effects = {
    'bold': (BOLD, CLR_BOLD),
    'italic': (ITALIC, CLR_ITALIC),
    'underline': (UNDERLINE, CLR_UNDERLINE),
    'inverse': (INVERSE, CLR_INVERSE),
}



from palett.structs import Card

grey = Card(
    base='#9E9E9E',
    lighten_5='#FAFAFA',
    lighten_4='#F5F5F5',
    lighten_3='#EEEEEE',
    lighten_2='#E0E0E0',
    lighten_1='#BDBDBD',
    darken_1='#757575',
    darken_2='#616161',
    darken_3='#424242',
    darken_4='#212121',
    accent_1='#C4C4C4',
    accent_2='#9E9E9E',
    accent_3='#858585',
    accent_4='#6B6B6B',
)



from palett.enum.color_cards import \
    RED, PINK, \
    PURPLE, DEEPPURPLE, \
    INDIGO, BLUE, LIGHTBLUE, CYAN, \
    TEAL, GREEN, \
    LIGHTGREEN, LIME, YELLOW, \
    AMBER, ORANGE, DEEPORANGE, \
    BROWN, BLUEGREY, GREY

red = [RED, PINK, ]
purple = [PURPLE, DEEPPURPLE, ]
blue = [INDIGO, BLUE, LIGHTBLUE, CYAN, ]
green = [TEAL, GREEN, ]
yellow_green = [LIGHTGREEN, LIME, YELLOW, ]
orange = [AMBER, ORANGE, DEEPORANGE, ]
grey = [BROWN, BLUEGREY, GREY]
rainbow = red + purple + blue + grey + green + yellow_green + orange
entire = rainbow + grey



from palett.structs import Card

teal = Card(
    base='#009688',
    lighten_5='#E0F2F1',
    lighten_4='#B2DFDB',
    lighten_3='#80CBC4',
    lighten_2='#4DB6AC',
    lighten_1='#26A69A',
    darken_1='#00897B',
    darken_2='#00796B',
    darken_3='#00695C',
    darken_4='#004D40',
    accent_1='#A7FFEB',
    accent_2='#64FFDA',
    accent_3='#1DE9B6',
    accent_4='#00BFA5'
)



from palett.structs import Card

deep_orange = Card(
    base='#FF5722',
    lighten_5='#FBE9E7',
    lighten_4='#FFCCBC',
    lighten_3='#FFAB91',
    lighten_2='#FF8A65',
    lighten_1='#FF7043',
    darken_1='#F4511E',
    darken_2='#E64A19',
    darken_3='#D84315',
    darken_4='#BF360C',
    accent_1='#FF9E80',
    accent_2='#FF6E40',
    accent_3='#FF3D00',
    accent_4='#DD2C00'
)



from palett.structs import Card

cyan = Card(
    base='#00BCD4',
    lighten_5='#E0F7FA',
    lighten_4='#B2EBF2',
    lighten_3='#80DEEA',
    lighten_2='#4DD0E1',
    lighten_1='#26C6DA',
    darken_1='#00ACC1',
    darken_2='#0097A7',
    darken_3='#00838F',
    darken_4='#006064',
    accent_1='#84FFFF',
    accent_2='#18FFFF',
    accent_3='#00E5FF',
    accent_4='#00B8D4'
)



from palett.structs import Card

red = Card(
    base='#F44336',
    lighten_5='#FFEBEE',
    lighten_4='#FFCDD2',
    lighten_3='#EF9A9A',
    lighten_2='#E57373',
    lighten_1='#EF5350',
    darken_1='#E53935',
    darken_2='#D32F2F',
    darken_3='#C62828',
    darken_4='#B71C1C',
    accent_1='#FF8A80',
    accent_2='#FF5252',
    accent_3='#FF1744',
    accent_4='#D50000'
)



from palett.structs import Card

light_green = Card(
    base='#8BC34A',
    lighten_5='#F1F8E9',
    lighten_4='#DCEDC8',
    lighten_3='#C5E1A5',
    lighten_2='#AED581',
    lighten_1='#9CCC65',
    darken_1='#7CB342',
    darken_2='#689F38',
    darken_3='#558B2F',
    darken_4='#33691E',
    accent_1='#CCFF90',
    accent_2='#B2FF59',
    accent_3='#76FF03',
    accent_4='#64DD17'
)



from palett.structs import Card

amber = Card(
    base='#FFC107',
    lighten_5='#FFF8E1',
    lighten_4='#FFECB3',
    lighten_3='#FFE082',
    lighten_2='#FFD54F',
    lighten_1='#FFCA28',
    darken_1='#FFB300',
    darken_2='#FFA000',
    darken_3='#FF8F00',
    darken_4='#FF6F00',
    accent_1='#FFE57F',
    accent_2='#FFD740',
    accent_3='#FFC400',
    accent_4='#FFAB00'
)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['palett',
 'palett.cards',
 'palett.convert',
 'palett.convert.utils',
 'palett.convert.utils.hex',
 'palett.convert.utils.hsl',
 'palett.convert.utils.rgb',
 'palett.dye',
 'palett.dye.color_to_ansi',
 'palett.dye.dye_factory',
 'palett.dye.hex',
 'palett.dye.rgb',
 'palett.dye.utils',
 'palett.enum',
 'palett.enum.color_cards',
 'palett.enum.color_degrees',
 'palett.enum.color_groups',
 'palett.enum.color_spaces',
 'palett.enum.font_effects',
 'palett.flopper',
 'palett.flopper.palett_flopper',
 'palett.fluo',
 'palett.fluo.fluo_entries',
 'palett.fluo.fluo_matrix',
 'palett.fluo.fluo_vector',
 'palett.presets',
 'palett.presets.rand_preset',
 'palett.projector',
 'palett.projector.utils',
 'palett.says',
 'palett.structs',
 'palett.structs.card',
 'palett.structs.preset',
 'palett.tabular',
 'palett.toner',
 'palett.utils',
 'palett.utils.ansi']

package_data = \
{'': ['*']}

install_requires = \
['aryth>=0.0.10',
 'crostab>=0.0.6',
 'intype>=0.0.2',
 'ject>=0.0.1',
 'pyspare>=0.0.6',
 'veho>=0.0.7']

setup_kwargs = {
    'name': 'palett',
    'version': '0.0.11',
    'description': 'pretty text for command line',
    'long_description': "## palett\n##### pretty text for command line\n\n### Usage\n```python\n\nfrom palett.fluo.fluo_vector import fluo_vector\nfrom palett.presets import FRESH, PLANET\n\nvectorCollection = [\n    [],\n    ['Xx', 'Yy', 'Zz', 'e', 'd', 'c', '-', '1', 2, 3],\n    [1, 1, 2, 3, 5, []],\n    ['a', 'b', 'c', 'd', 'e'],\n    ['beijing', 'shanghai', 'wuhan', 'xiamen', 'changsha']\n]\n\n\ndef test():\n    COMMA_SPACE = ', '\n    for vec in vectorCollection:\n        fluoed = fluo_vector(vec, (FRESH, PLANET))\n        print(f'[{COMMA_SPACE.join(fluoed)}]')\n\n\ntest()\n```",
    'author': 'Hoyeung Wong',
    'author_email': 'hoyeungw@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pydget/texting.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from palett.structs import Card

brown = Card(
    base='#795548',
    lighten_5='#EFEBE9',
    lighten_4='#D7CCC8',
    lighten_3='#BCAAA4',
    lighten_2='#A1887F',
    lighten_1='#8D6E63',
    darken_1='#6D4C41',
    darken_2='#5D4037',
    darken_3='#4E342E',
    darken_4='#3E2723',
    accent_1='#D2BEB6',
    accent_2='#B59387',
    accent_3='#A27767',
    accent_4='#855F51',
)



from .amber import amber
from .blue import blue
from .blue_grey import blue_grey
from .brown import brown
from .cyan import cyan
from .deep_orange import deep_orange
from .deep_purple import deep_purple
from .green import green
from .grey import grey
from .indigo import indigo
from .light_blue import light_blue
from .light_green import light_green
from .lime import lime
from .orange import orange
from .pink import pink
from .purple import purple
from .red import red
from .teal import teal
from .yellow import yellow



from random import randint, choice

from veho.vector.helper import swap

from palett.cards import grey
from palett.enum.color_degrees import entire
from palett.enum.color_groups import rainbow
from palett.enum.color_spaces import HEX
from palett.tabular import palett_crostab

LIGHTEN = 'lighten'
ACCENT = 'accent'
DARKEN = 'darken'


def degree_to_indice(degree: str):
    i = degree.find('_')
    if i < 0: return randint(14, 16)
    cate = degree[0: i]
    order = int(degree[(i + 1):])
    if cate == LIGHTEN: return 15 - (order - 1) * 3
    if cate == ACCENT: return 14 - (order - 1) * 3
    if cate == DARKEN: return 13 - (order - 1) * 3
    return randint(0, 16)


def palett_flopper(
        space=HEX,
        colors=rainbow,
        degrees=entire,
        default=grey.lighten_1,
        to=None,
        exhausted=True
):
    height, width = len(degrees), len(colors)
    crostab = palett_crostab(space, colors, degrees, dyed=False)
    degrees.sort(key=degree_to_indice, reverse=True)
    i = 0
    while i < height and (j := width):
        head, x = crostab.head[:], degrees[i]
        while (j := j - 1) >= 0:
            y = swap(head, randint(0, j), j)
            color = crostab.cell(x, y)
            yield to(color) if to else color
        i = i + 1
    color = default if default else crostab.cell(degrees[0], choice(crostab.head))
    default_color = to(color) if to else color
    while not exhausted: yield default_color
    return default_color



from palett.convert.rgb_int import rgb_int

'''
@param {[number,number,number]} rgb
@returns {string}
'''


# TODO: should convert to upper case

def rgb_hex(rgb): return f'#{rgb_int(rgb):06x}'

# return '#' + rgbToInt(rgb).toString(16).toUpperCase().padStart(6, '0')
